'use strict';

var express    = require('express');
var bodyParser = require('body-parser')
var cors       = require('cors');
var mongoose   = require('mongoose');

class API {

  constructor(props, resources) {
    this.props = props
    this.resources = resources
    this.schema = {}

    this.initialize()
  }

  model(resource) {
    return mongoose.model(resource.name, this.schema[resource.name])
  }

  initializeDB(callback) {
    mongoose.connect(this.props.dbUrl, callback);
  }

  initialize() {
    // Create the schema
    this.resources.forEach(resource => {
      var fields = {timestamp: { type: Date, default: Date.now }}
      resource.fields.map(field => {
        fields[field.name] = {
          type: field.type,
          required: [field.required, field.name + " field is required" ]
        }
      })
      this.schema[resource.name] = new mongoose.Schema(fields)
    })

    // Initialize the server
    this.app = express()
    this.app.use(bodyParser.json());
    this.app.use(bodyParser.urlencoded({extended: false}));
    this.app.use(cors());
  }

  mountResourceInterfaces(resource) {
    this.mountResourceRetrieveInterface(resource)
    this.mountResourceCreatorInterface(resource)
  }

  resourcePublicFields(resource) {
    var fields = ["timestamp"]
    resource.fields.forEach(field => {
      if (!field.private) {
        fields.push(field.name)
      }
    })
    return fields.join(" ")
  }

  mountResourceRetrieveInterface(resource) {
    var self = this
    this.app.get("/" + resource.name, cors(), function(req, res) {
      self.model(resource).find({}, self.resourcePublicFields(resource), function(err, data) {
        if(err) {
          return res.send({status: "fail", error: err});
        }

        return res.send({status: "success", [resource.name]: data});
      });
    });
  }

  mountResourceCreatorInterface(resource) {
    var self = this
    this.app.post("/" + resource.name, cors(), function(req, res) {
      self.model(resource).create(req.body.data, function(err, data) {
        if(err) {
          return res.send({status: "fail", error: err});
        }
        return res.send({status: "success", [resource.name]: data});
      });
    });
  }

  start(callback) {
    var self = this
    this.initializeDB(function(error){
      if (error) {
        callback(error)
        return;
      }

      // Mount the app resources
      self.resources.forEach(resource => {
        self.mountResourceInterfaces(resource)
      })

      self.app.listen(self.props.cloud.api.port, function () {
        callback()
      });
    });
  }

// function notify(config, options, callback) {
//   var mailOptions = {
//     from: config.general.email.from,
//     to: config.general.email.to,
//     subject: options.title
//   }
//
//   if (options.error) {
//     mailOptions.text = options.messageError;
//     mailOptions.html = "<b>" + options.messageError + "</b>";
//   } else if (options.data.data) {
//     mailOptions.text = "Details:\n\n";
//     mailOptions.html = "<b>Details:</b><br><br>";
//     Object.keys(options.data.data).map(function(option) {
//       mailOptions.text += option + ": " + options.data.data[option] + "\n\n";
//       mailOptions.html += option + ": <b>" + options.data.data[option] + "</b><br>";
//     });
//   }
//
//   utils.mailer.gmail(mailOptions, config.secure.gmail, function(response, error) {
//     if (error) {
//       callback(error);
//       return;
//     }
//
//     callback();
//   });
// }

}

module.exports = API;
